<?php

/***************************************************************
*  Copyright notice
*
*  (c) 2005 Kristian Domke (kristian@neither-nor.net)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * Plugin 'Photo Gallery (DAM)' for the 'kdo_dam_photogallery' extension.
 *
 * @author	Kristian Domke <kristian@neither-nor.net>
 */

require_once (PATH_tslib.'class.tslib_pibase.php');

class tx_kdodamphotogallery_pi1 extends tslib_pibase {
	var $prefixId = 'tx_kdodamphotogallery_pi1'; // Same as class name
	var $scriptRelPath = 'pi1/class.tx_kdodamphotogallery_pi1.php'; // Path to this script relative to the extension dir.
	var $extKey = 'kdo_dam_photogallery'; // The extension key.
	var $table = 'tx_kdodamphotogallery_categorie';
	var $conf = array ();
	var $dataPages;
	var $piVars;
	var $categories = array ();

	//Only for Development:
	var $LLtestPrefix = '';
	var $LLtestPrefixAlt = '#missingLabel#';

	/**
	 * Here the story begins...
	 *
	 * @param	array		The $content Array
	 * @param	array		The $conf Array
	 * @return	string		The content
	 */
	function main($content, $conf) {
		// Getting configuration:
		$this->conf = $conf;
		$this->pi_setPiVarDefaults();

		// Loading localization data:
		$this->pi_loadLL();

		// Init Flexform configuration:
		$this->pi_initPIFlexForm();

		// Where are our Categories?
		$this->dataPages = $this->getDataPages();

		//Cache control:
		$this->pi_checkCHash = TRUE;
		$this->pi_USER_INT_obj = 0;
		$this->pi_autoCacheEn = 1;
		$this->pi_autoCacheFields = array (
			'cat' => array ('range' => array (0, 9999)), 
			'img' => array ('range' => array (0, 9999)), 
			'page' => array ('range' => array (0, 9999)),
		);
		$this->pi_checkCHash = TRUE;

		// Which side shall we show?
		if (isset ($this->piVars['img']) && isset ($this->piVars['cat'])) {
			$content .= $this->renderSingleView();
		}
		elseif (isset ($this->piVars['cat'])) {
			$content .= $this->renderThumpView();
		} else {
			$content .= $this->renderOverview();
		}

		$content .= '<p>'.date('Y-m-d H:i:s').'</p>';

		return $this->pi_wrapInBaseClass($content);

	} // end function main()

	/**
	 * Renders the overview over the whole photo gallery
	 *
	 * @return	string		Content ready to be returned by the main method
	 */
	function renderOverView() {

		/*
		 * Start Rendering
		 */

		// Initialize local cObject:
		$lCObj = t3lib_div :: makeInstance('tslib_cObj');
		$lCObj->setParent($this->cObj->data, $this->cObj->currentRecord);

		// Get header text from FlexForm and set it in the $conf-Array:
		$headerText = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'headerText', 'overView');
		$this->conf['overViewHeader.']['99.']['value'] = htmlspecialchars($headerText);

		// Put the cObject together:
		$this->conf['overViewObj.']['marks.']['HEADER.'] = $this->conf['overViewHeader.'];
		$this->conf['overViewObj.']['marks.']['CATEGORYMENU.'] = $this->conf['overViewMenu.'];

		// Render the object and return the code:
		$content = $lCObj->cObjGetSingle($this->conf['overViewObj'], $this->conf['overViewObj.']);

		return $content;

	} //end function renderOverView

	/**
	 * Renders the overview over one categorie of the photo gallery
	 *
	 * @return	string		Content ready to be returned by the main method
	 */
	function renderThumpView() {

		//Initialize ObjectData:
		$this->fetchCategories();

		// Get configuration data from FlexForm:
		$tableWidth = intval($this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'thumpWidth', 'thumpView'));
		$tableWidth = ($tableWidth >= 100 && $tableWidth <= 1000) ? $tableWidth : 400;

		$thumpCols = intval($this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'thumpCol', 'thumpView'));
		$thumpCols = $thumpCols ? $thumpCols : 3;

		$thumpRows = intval($this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'thumpRow', 'thumpView'));
		$thumpRows = $thumpRows ? $thumpRows : 3;

		$thumpsPerPage = $thumpCols * $thumpRows;

		/*
		 * Which images are in this category?
		 */

		// Which Category?
		$catUID = intval($this->piVars['cat']);
		if (!isset ($catUID) || $catUID <= 0 || !isset ($this->categories[$catUID])) {
			// The category is not valid!
			// TODO: Implement Meta-Refresh to the Main Gallery-Page
		}

		// Get the category out of the DB:
			$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', //SELECT ...
		$this->table, //FROM ...
		'uid = '.$catUID.$this->cObj->enableFields($this->table), //WHERE ...
		'', //GROUP BY ...
		'', //ORDER BY ...
		'' //LIMIT
	);

		if (!isset ($res)) {
			// We did not get a valid result. Break here
			// TODO: Implement Meta-Refresh to the Main Gallery-Page
		}
		$category = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);

		// Set the Description of the category into the Header:
		$this->conf['thumpViewHeader.']['99.']['value'] = $category['description'];

		//We have to clean the array, to be sure to show only the active ones:
		$imageUID = $this->cleanUIDarray($category['images'], 'tx_dam');

		//Now slice out the records for this page (is none is specified, page = 1)
		$page = (int) $this->piVars['page'] > 1 ? (int) $this->piVars['page'] : 1;
		$firstImage = ($page -1) * $thumpsPerPage;
		$imageUID = array_slice($imageUID, $firstImage, $thumpsPerPage);

		//pad the array to the next full row:
		while (sizeof($imageUID) % $thumpCols != 0) {
			$imageUID[] = 0;
		}

		/*
		 * Now we should have all the Data we need.
		 * Begin Rendering!
		 */

		//get the Object for each thumpnail from the TypoScript Template
		$thumpItem = $this->conf['thumpViewThump.'];

		// How large may a thumpnail be?
		$thumpWidth = ($tableWidth / $thumpCols) - 20;
		//Set the maximum size into the $thumpItem:
		$thumpItem['file.']['maxW'] = $thumpWidth;
		$thumpItem['file.']['maxH'] = $thumpWidth;

		//Initalize first row:
		$rawHTML .= '<tr style="height:'. ($thumpWidth +20).'px;">';

		//Start iterating over the Images
		foreach ($imageUID as $uid) {

			//initialize counter:
			if (!isset ($inRow)) {
				$inRow = 1;
			}
			if (!isset ($Rows)) {
				$Rows = 1;
			}

			if ($uid > 0) {

				//get the imageinfo from DB:
					$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', //SELECT ...
		'tx_dam', //FROM ...
		'uid = '.$uid.$this->cObj->enableFields('tx_dam'), //WHERE ...
		'', //GROUP BY ...
		'', //ORDER BY ...
		'' //LIMIT
	);

				$image = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);

				$imagePath = $this-> $BACK_PATH.$image['file_path'].$image['file_name'];
				$altTag = $this->pi_getLL('thumpView_photograph').$image['creator'];

				$rawHTML .= $this->buildThump($thumpItem, $imagePath, $uid, $altTag);
			} else {
				$rawHTML .= '<td class="tx_kdodamphotogallery_pi1_tv_thump">&nbsp;</td>';
			}

			if ($inRow == $thumpCols && ($Rows == $thumpRows || $uid == 0)) {
				//The page is full or we have no more records, end it here!
				$rawHTML .= '</tr>'."\n";
			}
			elseif ($inRow == $thumpCols && $Rows < $thumpRows) {
				//The row is full, start the next one!
				$rawHTML .= '</tr>'."\n".'<tr style="height:'. ($thumpWidth +20).'px;">';
				$inRow = 1;
				$Rows ++;
			} else {
				$inRow ++;
			}

		}

		//

		//now lets puzzle the whole Object:
		$this->conf['thumpViewObj.']['marks.']['THUMPNAILS.']['value'] = $rawHTML;
		$this->conf['thumpViewObj.']['marks.']['HEADER.'] = $this->conf['thumpViewHeader.'];
		$this->conf['thumpViewObj.']['marks.']['MENU.'] = $this->conf['thumpViewMenu.'];

		//initialize a local cObj and render the output:
		$lCObj = t3lib_div :: makeInstance('tslib_cObj');
		$lCObj->setParent($this->cObj->data, $this->cObj->currentRecord);

		$content = $lCObj->cObjGetSingle($this->conf['thumpViewObj'], $this->conf['thumpViewObj.']);

		return $content;
	} // end function renderCategoryView()

	/**
	 * Renders the view of one photo in the gallery
	 *
	 * @return	string		Content ready to be returned by the main method
	 */
	function renderSingleView() {
		//Getting the parts to work with:
		$imageObj = $this->conf['singleViewImage.'];
		$metaObj = $this->conf['singleViewMetadata.'];
		$menuObj = $this->conf['singleViewMenu.'];

		/*
		 * Generate the ImageObject:
		 */

		//Get all important Data from the Flexform:
		$showCopyright = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'singleShowCR', 'singleView') ? true : false;
		$imageWidth = intval($this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'imageWidth', 'singleView'));
		$imageWidth = ($imageWidth >= 100 && $imageWidth <= 1000) ? $imageWidth : 400;
		$copyrightText = htmlspecialchars($this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'copyCRtext', 'copyRight'));
		$copyrightOpacity = intval($this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'copyOpacity', 'copyRight'));
		$copyrightOpacity = ($copyrightOpacity > 0 && $copyrightOpacity <= 100) ? $copyrightOpacity : 70;
		$copyrightBGcolor = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'copyBGcolor', 'copyRight');

		//get Information about the Category:
			$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('uid, images', //SELECT ...
		$this->table, //FROM ...
		'uid = '.$this->piVars['cat'].$this->cObj->enableFields($this->table), //WHERE ...
		'', //ORDER BY ...
		'' //LIMIT ...
	);
		$catInfo = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);
		$imageUID = $this->cleanUIDarray($catInfo['images'], 'tx_dam');

		//get Information about the Image:
			$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', //SELECT ...
		'tx_dam', //FROM ...
		'uid = '.$this->piVars['img'].$this->cObj->enableFields('tx_dam'), //WHERE ...
		'', //ORDER BY ...
		'' //LIMIT ...
	);
		$imageInfo = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);

		//Check, if Category and Image are valid:
		if (!(in_array($this->piVars['img'], $imageUID))) {
			return NULL;
		}

		//calculate imageData:
		$imageSize = getimagesize($imageInfo['file_path'].$imageInfo['file_name']);
		$imageInfo['aspectRatio'] = $imageSize[0] / $imageSize[1];

		if ($imageInfo['aspectRatio'] >= 1) {
			//image in landscape format (or square)
			$width = $imageWidth;
			$height = intval($imageWidth / $imageInfo['aspectRatio']);
		} else {
			//image in portrait format
			$width = $imageWidth * $imageInfo['aspectRatio'];
			$height = $imageWidth;
		}
		//Set the width/height for the IMAGE:
		$imageObj['file.']['width'] = $width;
		$imageObj['file.']['height'] = $height;

		//Set it to the right file:
		$imageObj['file'] = $imageInfo['file_path'].$imageInfo['file_name'];

		//Do we have a copyright-stamp?
		if ($showCopyright && $copyrightText) {

			//Set the background of the stamp:
			$imageObj['file.']['m.']['bgImg.']['backColor'] = $copyrightBGcolor ? $copyrightBGcolor : '#dddddd';
			$imageObj['file.']['m.']['bgImg.']['XY'] = $width.','.$height;

			//Set the copyright text:
			$imageObj['file.']['m.']['bgImg.']['10.']['text'] = $copyrightText;
			$imageObj['file.']['m.']['bgImg.']['10.']['offset'] = ($width -5).','. ($height -5);

			//Calculate the color for the opacity from percentage to HTML-color:
			if ($copyrightOpacity < 0 || $copyrightOpacity > 100) {
				$copyrightOpacity = 0;
			}
			$opacity = ((100 - $copyrightOpacity) / 100 * 255);
			$opacity = sprintf('#%02x%02x%02x', $opacity, $opacity, $opacity);

			//Create the mask:
			$imageObj['file.']['m.']['mask.']['backColor'] = '#ffffff';
			$imageObj['file.']['m.']['mask.']['XY'] = $width.','.$height;
			$imageObj['file.']['m.']['mask.']['10.']['dimensions'] = ($width -10).' - [20.w]'.','. ($height -10).'- [20.h]'.',[20.w] + 10,[20.h] + 10';
			$imageObj['file.']['m.']['mask.']['10.']['color'] = $opacity;
			$imageObj['file.']['m.']['mask.']['20.'] = $imageObj['file.']['m.']['bgImg.']['10.'];
		} else {
			unset ($imageObj['file.']['m.']);
		}

		/*
		 * Generate MetaData Object
		 */

		//Get all important Data from the Flexform:
		$metaShowTitle = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'singleShowTitle', 'singleView') ? true : false;
		$metaShowDesc = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'singleShowDesc', 'singleView') ? true : false;
		$metaShowCreator = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'singleShowCreator', 'singleView') ? true : false;
		if ($metaShowTitle) {
			$title = $this->pi_getLL('singleView_title');
			$title .= htmlspecialchars($imageInfo['title']);
			$metaObj['99.']['99.']['value'] = $title;
		} else {
			unset ($metaObj['99.']['99']);
			unset ($metaObj['99.']['99.']);
		}

		if ($metaShowDesc) {
			$desc = $this->pi_getLL('singleView_desc');
			$desc .= ereg("\n", '<br />', $desc);
			$desc .= htmlspecialchars($imageInfo['description']);
			$metaObj['99.']['199.']['value'] = $desc;
		} else {
			unset ($metaObj['99.']['199']);
			unset ($metaObj['99.']['199.']);
		}

		if ($metaShowCreator) {
			$creator = $this->pi_getLL('singleView_creator');
			$creator .= htmlspecialchars($imageInfo['creator']);
			$metaObj['99.']['299.']['value'] = $creator;
		} else {
			unset ($metaObj['99.']['299']);
			unset ($metaObj['99.']['299.']);
		}

		/*
		 * Generate the BackLink:
		 */

		// Calculate, on which page we have to link:
		$thumpCols = intval($this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'thumpCol', 'thumpView'));
		$thumpCols = $thumpCols ? $thumpCols : 3;
		$thumpRows = intval($this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'thumpRow', 'thumpView'));
		$thumpRows = $thumpRows ? $thumpRows : 3;
		$thumpsPerPage = $thumpCols * $thumpRows;

		$activePage = ceil((array_search($this->piVars['img'], $imageUID) + 1) / $thumpsPerPage);

		//Add the parameters to the link:
		$this->conf['singleViewObj.']['marks.']['LINKBACK.']['typolink.']['additionalParams'] = '&tx_kdodamphotogallery_pi1[cat]='.$this->piVars['cat'].'&tx_kdodamphotogallery_pi1[page]='.$activePage;

		/*
		 *  Put all things together and render the content:
		 */

		$this->conf['singleViewObj.']['marks.']['IMAGE.'] = $imageObj;
		$this->conf['singleViewObj.']['marks.']['METADATA.'] = $metaObj;
		$this->conf['singleViewObj.']['marks.']['MENU.']['99.']['1.'] = $menuObj;

		$lCObj = t3lib_div :: makeInstance('tslib_cObj');
		$lCObj->setParent($this->cObj->data, $this->cObj->currentRecord);
		$content = $lCObj->cObjGetSingle($this->conf['singleViewObj'], $this->conf['singleViewObj.']);

		return $content;
	} // end function renderSingleView()

	/**
	 * Reads the Categories, that should be shown from the database and inserts
	 * them into $this->categories. The array is also returned by this method
	 *
	 * @return	array
	 */
	function fetchCategories() {

		// Where are our Categories?
		$pages = $this->getDataPages();

		// Lets get the Cats:
			$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', //SELECT ...
		$this->table, //FROM ...
		'pid IN ('.$pages.')'.$this->cObj->enableFields($this->table), //WHERE ...
		'', //GROUP BY ...
		'sorting', //ORDER BY ...
		'' //LIMIT
	);

		// Build $this->categories array:
		$i = 0;
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res)) {
			$this->categories[$row['uid']] = $row;
		}

		return $this->categories;

	} // end function fetchCategories()

	/**
	 * Gets the uids of the Pages on which the plugin data could be stored. It
	 * returns an array of the uids
	 * TODO: Enable Recursion!
	 *
	 * @return	array		UIDs of the data Pages
	 */
	function getDataPages() {
		$ret = array ();
		// Do we have a special data page?
		if (isset ($this->cObj->data['pages']) && intval($this->cObj->data['pages']) != '0') {
			$ret = $this->cObj->data['pages'];
		} else {
			$ret = $GLOBALS['TSFE']->id;
		} # end if

		return $ret;

	} // end function getDataPages()

	/**
	 * Generates the html-code for a thumpnail, using the TS-Template in $this->conf
	 *
	 * @param	string		Path to the ImageFile
	 * @param	string		String to use as AltTag
	 * @param	integer		DAM-UID of the Image. Used for linking.
	 * @param	array		TS-array for gerenarting the object
	 * @return	string		HTML-Code
	 */
	function buildThump($thumpItem, $path, $uid, $altTag) {
		//Put the Values into the $this->conf array
		$thumpItem['file'] = $path;
		$thumpItem['altText'] = $altTag;
		$thumpItem['titleText'] = $altTag;

		//Shall we link to the single View?
		if (($this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'thumpLinkSV', 'thumpView'))) {
			$thumpItem['stdWrap.']['typolink.']['parameter'] = $GLOBALS['TSFE']->id;
			$thumpItem['stdWrap.']['typolink.']['additionalParams'] = '&tx_kdodamphotogallery_pi1[cat]='.$this->piVars['cat'].'&tx_kdodamphotogallery_pi1[img]='.$uid;
			$thumpItem['stdWrap.']['typolink.']['useCacheHash'] = '1';
		}

		// Initialize local cObject:
		$lCObj = t3lib_div :: makeInstance('tslib_cObj');
		$lCObj->setParent($this->cObj->data, $this->cObj->currentRecord);

		// Render the HTML and return it:
		return $lCObj->cObjGetSingle('IMAGE', $thumpItem);

	}

	/**
	 * Returns an array of the valid UIDs in the right Order for further processing
	 *
	 * @param	string		UIDs seperated by ','
	 * @param	string		Name of the table to look in
	 * @return	array		Sorted Array of UIDs
	 */
	function cleanUIDarray($UIDs, $tableName) {
		$tmpUIDarray = array (); // temporary Data
		$rawUIDarray = t3lib_div :: intExplode(',', $UIDs); //The given UIDs as array
		$retUIDarray = array (); // array to be returned

		//Select the right records from the table:
			$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('uid', //SELECT ...
		$tableName, //FROM ...
		'uid IN ('.$UIDs.')'.$this->cObj->enableFields($tableName), //WHERE ...
		'', //GROUP BY ...
		'', //ORDER BY ...
		'' //LIMIT
	);

		while ($record = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res)) {
			$tmpUIDarray[$record['uid']] = true;
		}

		foreach ($rawUIDarray as $UID) {
			if ($tmpUIDarray[$UID]) {
				$retUIDarray[] = $UID;
			}
		}

		return $retUIDarray;
	}

	/**
	 * Calculates the cHash and adds it to the string.
	 *
	 * @param	array	Parameter Array to be overwritten / appended
	 * @param	boolean	if true, return complete URL
	 * @return	string	
	 */
	function buildGetString($paramArray, $withURL = FALSE) {

		$url = $this->pi_linkTP_keepPIvars_url($paramArray,TRUE);

		if (!$withURL) {
			list ($page, $paramString) = split('\?', $url);
		}

		return $paramString;
	}

	/**
	 * Generates e $menuItemsArray for the userfunction included by the static
	 * Template.  It is used for overview.
	 *
	 * @param	array		The $content array
	 * @param	array		The $conf array
	 * @return	array		Returns the multidimensional array of the menuitems.
	 */
	function generateOverViewMenu($content, $conf) {

		// It is not possible to use the methods above, because the class-hirarchy
		// is different. So we have to do it manually:

		//Get the data pages:
		$pages = array ();
		if (isset ($this->cObj->parentRecord['data']['pages']) && intval($this->cObj->parentRecord['data']['pages']) != '0') {
			$pages = $this->cObj->parentRecord['data']['pages'];
		} else {
			$pages = $GLOBALS['TSFE']->id;
		} # end if

		//Fetch the Categories:
			$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', //SELECT ...
		$this->table, //FROM ...
		'pid IN ('.$pages.')'.$this->cObj->enableFields($this->table), //WHERE ...
		'', //GROUP BY ...
		'sorting', //ORDER BY ...
		'' //LIMIT
	);

		//Where are we?
		$activeCat = $piVars['cat'];

		// Build the $menuItemsArray array:
		$menuItemsArray = array ();
		while ($item = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res)) {
			if ($item['uid'] == $activeCat) {
				$addQueryParams = array (
					'cat' => $item['uid'],
					'page' => '1'
				);
				$menuItemsArray[] = array ('title' => $item['title'], 'uid' => $GLOBALS['TSFE']->id, '_ADD_GETVARS' => $this->buildGetString($addQueryParams), 'ITEM_STATE' => 'ACT');
			} else {
				$addQueryParams = array (
					'cat' => $item['uid'],
					'page' => '1'
				);
				$menuItemsArray[] = array ('title' => $item['title'], 'uid' => $GLOBALS['TSFE']->id, '_ADD_GETVARS' => $this->buildGetString($addQueryParams), 'ITEM_STATE' => 'NO');
			}
		}

		return $menuItemsArray;
	}

	/**
	 * Generates the $menuItemsArray for the userfunction included by the static
	 * Template. It is used for thumpview.
	 *
	 * @param	array		The $content array
	 * @param	array		The $conf array
	 * @return	array		Returns the multidimensional array of the menuitems.
	 */
	function generateThumpViewMenu($content, $conf) {

		//initalize Data:
		$this->pi_setPiVarDefaults();

		// It is not possible to use the methods above, because the class-hirarchy
		// is different. So we have to do it manually:

		//get our Data:
		$thumpCols = intval($this->pi_getFFvalue($this->cObj->parentRecord['data']['pi_flexform'], 'thumpCol', 'thumpView'));
		$thumpCols = $thumpCols ? $thumpCols : 3;
		$thumpRows = intval($this->pi_getFFvalue($this->cObj->parentRecord['data']['pi_flexform'], 'thumpRow', 'thumpView'));
		$thumpRows = $thumpRows ? $thumpRows : 3;
		$thumpsPerPage = $thumpCols * $thumpRows;

		//Get the data pages:
		$pages = array ();
		if (isset ($this->cObj->parentRecord['data']['pages']) && intval($this->cObj->parentRecord['data']['pages']) != '0') {
			$pages = $this->cObj->parentRecord['data']['pages'];
		} else {
			$pages = $GLOBALS['TSFE']->id;
		} # end if

		//Get the current Category and Page:
		$activeCat = intval($this->piVars['cat']);
		$activePage = intval($this->piVars['page']) ? intval($this->piVars['page']) : '1';

		//Is the category valid?
			$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('uid, images', //SELECT ...
		$this->table, //FROM ...
		'uid = '.$activeCat.$this->cObj->enableFields($this->table), //WHERE ...
		'', //GROUP BY ...
		'', //ORDER BY ...
		'' //LIMIT
	);

		$activeCat = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);

		if (!isset ($activeCat['uid'])) {
			return NULL;
		}

		//Get the array of all the Images in this category:
		$imageUID = $this->cleanUIDarray($activeCat['images'], 'tx_dam');

		$numPages = ceil(sizeof($imageUID) / $thumpsPerPage);

		// Build the $menuItemsArray array:
		$menuItemsArray = array ();

		if ($activePage != 1) {
			//We are not on the first Page
			$addQueryParams = array ('page' => ($activePage -1));
			$menuItemsArray[] = array ('title' => '<<', 'uid' => $GLOBALS['TSFE']->id, '_ADD_GETVARS' => $this->buildGetString($addQueryParams), 'ITEM_STATE' => 'NO');
		} else {
			//We _are_ on the first Page
			$addQueryParams = array ();
			$menuItemsArray[] = array ('title' => '<<', 'uid' => $GLOBALS['TSFE']->id, '_ADD_GETVARS' => $this->buildGetString($addQueryParams), 'ITEM_STATE' => 'ACT');
		}

		for ($i = 1; $i <= $numPages; $i ++) {
			if ($i == $activePage) {
				$addQueryParams = array ('page' => $i);

				$menuItemsArray[] = array ('title' => $i, 'uid' => $GLOBALS['TSFE']->id, '_ADD_GETVARS' => $this->buildGetString($addQueryParams), 'ITEM_STATE' => 'ACT');
			} else {
				$addQueryParams = array ('page' => $i);
				$menuItemsArray[] = array ('title' => $i, 'uid' => $GLOBALS['TSFE']->id, '_ADD_GETVARS' => $this->buildGetString($addQueryParams), 'ITEM_STATE' => 'NO');
			}
		}

		if ($activePage != $numPages) {
			//We are not on the last Page
			$addQueryParams = array('page' => ($activePage + 1));
			$menuItemsArray[] = array ('title' => '>>', 'uid' => $GLOBALS['TSFE']->id, '_ADD_GETVARS' => $this->buildGetString($addQueryParams), 'ITEM_STATE' => 'NO');
		} else {
			//We _are_ on the last Page
			$addQueryParams = array();
			$menuItemsArray[] = array ('title' => '>>', 'uid' => $GLOBALS['TSFE']->id, '_ADD_GETVARS' => $this->buildGetString($addQueryParams), 'ITEM_STATE' => 'ACT');
		}

		return $menuItemsArray;
	}
	/**
	 * Generates the $menuItemsArray for the userfunction included by the static
	 * Template. It is used for singleView.
	 *
	 * @param	array		The $content array
	 * @param	array		The $conf array
	 * @return	array		Returns the multidimensional array of the menuitems.
	 */
	function generateSingleViewMenu($content, $conf) {

		//initalize Data:
		$this->pi_setPiVarDefaults();

		// It is not possible to use the methods above, because the object-hirarchy
		// is different. So we have to do it manually:

		//Get the data pages:
		$pages = array ();
		if (isset ($this->cObj->parentRecord['data']['pages']) && intval($this->cObj->parentRecord['data']['pages']) != '0') {
			$pages = $this->cObj->parentRecord['data']['pages'];
		} else {
			$pages = $GLOBALS['TSFE']->id;
		} # end if

		//Get the current Category and Image:
		$activeCat = intval($this->piVars['cat']);
		$activeImage = intval($this->piVars['img']);

		//Are category and image valid?
			$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('uid, images', //SELECT ...
		$this->table, //FROM ...
		'uid = '.$activeCat.$this->cObj->enableFields($this->table), //WHERE ...
		'', //GROUP BY ...
		'', //ORDER BY ...
		'' //LIMIT
	);

		$activeCat = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);

		//Get the array of all the Images in this category:
		$imageUID = array ();
		$imageUID = $this->cleanUIDarray($activeCat['images'], 'tx_dam');

		if (!(isset ($activeCat['uid']) && in_array($activeImage, $imageUID))) {
			return NULL;
		}

		// Build the $menuItemsArray array:
		$menuItemsArray = array ();
		if ($activeImage != $imageUID[0]) {
			//We are not on the first Page
			$addQueryParams = array('img' => ($imageUID[array_search($activeImage, $imageUID) - 1]));
			$menuItemsArray[] = array ('title' => '<<', 'uid' => $GLOBALS['TSFE']->id, '_ADD_GETVARS' => $this->buildGetString($addQueryParams), 'ITEM_STATE' => 'NO');
		} else {
			//We _are_ on the first Page
			$addQueryParams = array();
			$menuItemsArray[] = array ('title' => '<<', 'uid' => $GLOBALS['TSFE']->id, '_ADD_GETVARS' => $this->buildGetString($addQueryParams), 'ITEM_STATE' => 'ACT');
		}

		//Generate the info about which image we're on:
		$addQueryParams = array();
		$menuItemsArray[] = array ('title' => array_search($activeImage, $imageUID) + 1, 'uid' => $GLOBALS['TSFE']->id, '_ADD_GETVARS' => $this->buildGetString($addQueryParams), 'ITEM_STATE' => 'ACT');

		if ($activeImage != $imageUID[count($imageUID) - 1]) {
			//We are not on the last Page
			$addQueryParams = array('img' => ($imageUID[array_search($activeImage, $imageUID) + 1]));
			$menuItemsArray[] = array ('title' => '>>', 'uid' => $GLOBALS['TSFE']->id, '_ADD_GETVARS' => $this->buildGetString($addQueryParams), 'ITEM_STATE' => 'NO');
		} else {
			//We _are_ on the last Page
			$addQueryParams = '&tx_kdodamphotogallery_pi1[cat]='.$activeCat['uid'].'&tx_kdodamphotogallery_pi1[img]='. ($activeImage);
			$menuItemsArray[] = array ('title' => '>>', 'uid' => $GLOBALS['TSFE']->id, '_ADD_GETVARS' => $this->buildGetString($addQueryParams), 'ITEM_STATE' => 'ACT');
		}

		return $menuItemsArray;
	}

}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/kdo_dam_photogallery/pi1/class.tx_kdodamphotogallery_pi1.php']) {
	include_once ($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/kdo_dam_photogallery/pi1/class.tx_kdodamphotogallery_pi1.php']);
}
?>

